/* ***** BEGIN LICENSE BLOCK ***** 
 * Version: RCSL 1.0/RPSL 1.0 
 *  
 * Portions Copyright (c) 1995-2002 RealNetworks, Inc. All Rights Reserved. 
 *      
 * The contents of this file, and the files included with this file, are 
 * subject to the current version of the RealNetworks Public Source License 
 * Version 1.0 (the "RPSL") available at 
 * http://www.helixcommunity.org/content/rpsl unless you have licensed 
 * the file under the RealNetworks Community Source License Version 1.0 
 * (the "RCSL") available at http://www.helixcommunity.org/content/rcsl, 
 * in which case the RCSL will apply. You may also obtain the license terms 
 * directly from RealNetworks.  You may not use this file except in 
 * compliance with the RPSL or, if you have a valid RCSL with RealNetworks 
 * applicable to this file, the RCSL.  Please see the applicable RPSL or 
 * RCSL for the rights, obligations and limitations governing use of the 
 * contents of the file.  
 *  
 * This file is part of the Helix DNA Technology. RealNetworks is the 
 * developer of the Original Code and owns the copyrights in the portions 
 * it created. 
 *  
 * This file, and the files included with this file, is distributed and made 
 * available on an 'AS IS' basis, WITHOUT WARRANTY OF ANY KIND, EITHER 
 * EXPRESS OR IMPLIED, AND REALNETWORKS HEREBY DISCLAIMS ALL SUCH WARRANTIES, 
 * INCLUDING WITHOUT LIMITATION, ANY WARRANTIES OF MERCHANTABILITY, FITNESS 
 * FOR A PARTICULAR PURPOSE, QUIET ENJOYMENT OR NON-INFRINGEMENT. 
 * 
 * Technology Compatibility Kit Test Suite(s) Location: 
 *    http://www.helixcommunity.org/content/tck 
 * 
 * Contributor(s): 
 *  
 * ***** END LICENSE BLOCK ***** */ 

#ifndef __RLSTATA_H__
#define __RLSTATE_H__

class CASMRuleState
{
private:
    UINT16  m_nNumRules;
    BOOL*   m_bNeedSwitchOff;
    BOOL*   m_bSubscribePending;
    BOOL*   m_bSubscribed;
    BOOL*   m_bUnsubscribePending;
    UINT8*  m_lastASMFlagsForThisRule;
    UINT16** m_OffDepends;
    UINT16** m_OnDepends;

    void ParseRuleBookForDirectives(UINT16 num_rules,
				    char* pRuleBook,
				    UINT16 usBookSize,
				    BOOL* pNeedSwitchOff,
				    UINT16** pOffDepends,
				    UINT16** pOnDepends);

    char* ParseDependsList(char* pDependsText, UINT16* pDependsList);

public:
    CASMRuleState(UINT16 nNumRules, char* pRuleBook, UINT16 usBookSize);
    ~CASMRuleState();

    void StartSubscribePending(UINT16 usRuleNum);
    BOOL CanSubscribeNow(UINT16 usRuleNum);
    void CompleteSubscribe(UINT16 usRuleNum);

    void StartUnsubscribePending(UINT16 usRuleNum);
    BOOL CanUnsubscribeNow(UINT16 usRuleNum);
    void CompleteUnsubscribe(UINT16 usRuleNum);

    BOOL CanSwitchStreamsNow();
    void CompleteStreamSwitch();
    void CancelStreamSwitch();
    void CompleteAllUnsubscribes();

    BOOL AnyPendingUnsubscribes();
    UINT16 GetNextPendingUnsubscribe();

    BOOL IsRuleSubscribed(UINT16 usRuleNum) 
	{return m_bSubscribed[usRuleNum];}
    BOOL IsUnsubscribePending(UINT16 usRuleNum)
	{return m_bUnsubscribePending[usRuleNum];}
    BOOL IsSubscribePending(UINT16 usRuleNum)
	{return m_bSubscribePending[usRuleNum];}
    void SetASMFlags(UINT16 usRuleNum, UINT8 asmFlags)
	{m_lastASMFlagsForThisRule[usRuleNum] = asmFlags;}
    UINT8 GetASMFlags(UINT16 usRuleNum)
	{return m_lastASMFlagsForThisRule[usRuleNum];}
    BOOL WaitForSwitchOff(UINT16 usRuleNum)
	{return m_bNeedSwitchOff[usRuleNum];}
};
#endif /* __RLSTATE_H__ */
